"""A collection of useful functions"""
from hawkweed.functional.primitives import *
from hawkweed.functional.logical import *
from hawkweed.functional.mathematical import *
from hawkweed.functional.list_prims import *
