"A collection of augmented standard classes"
from hawkweed.classes.list_cls import List
from hawkweed.classes.dict_cls import Dict
from hawkweed.classes.iterable import Iterable
from hawkweed.classes.set_cls import Set
from hawkweed.classes.repr import Repr
from hawkweed.classes.collection import Collection
