"""A few monads"""
from hawkweed.monads.monad import Monad
from hawkweed.monads.listm import ListM
from hawkweed.monads.continuation import Continuation
from hawkweed.monads.identity import Identity
from hawkweed.monads.maybe import *
from hawkweed.monads.io import IO, CachedIO
from hawkweed.monads.either import either
